#pragma once

#include "Morale.h"

#include <array>
#include <vector>

class Population
{
public:
	enum PersonRole
	{
		ROLE_CHILD,
		ROLE_STUDENT,
		ROLE_WORKER,
		ROLE_SCIENTIST,
		ROLE_RETIRED
	};

public:
	Population();
	~Population();

	uint32_t size();
	uint32_t size(PersonRole);

	uint32_t birthCount() const { return mBirthCount; }
	uint32_t deathCount() const { return mDeathCount; }

	void clear();

	void populateList(PersonRole _role, uint32_t count);

	int update(int morale, int food, int universities = 0, int nurseries = 0, int hospitals = 0);

	void seedPopulation();

	void starveRate(float r) { mStarveRate = r; }

protected:

private:
	void init();
	void clearPopulationList(PersonRole);

	void spawn_children(int morale, int nurseries);
	void spawn_students();
	void spawn_adults(int universities);
	void spawn_retiree();

	void kill_children(int morale, int nurseries);
	void kill_students(int morale, int hospitals);
	void kill_adults(Population::PersonRole role, int morale, int hospitals);

	uint32_t consume_food(uint32_t _food);

private:
	typedef std::array<uint32_t, 5>		PopulationTable;
	typedef std::array<MoraleModifier, 5> MoraleModifiers;

private:
	uint32_t			mBirthCount;				/**<  */
	uint32_t			mDeathCount;				/**<  */

	float				mStarveRate;				/**< Amount of population that dies during food shortages in percent. */

	PopulationTable		mPopulation;				/**< Current population. */
	PopulationTable		mPopulationGrowth;			/**< Population growth table. */
	PopulationTable		mPopulationDeath;			/**< Population death table. */

	MoraleModifiers		mModifiers;					/**< Morale modifier table */

	bool				mPopulationCanMakeBabies;	/**< Flag indicating that enough time has passed for it to make sense for babies to be born. */
};
